"""A Python program to iterate through a directory and create CASTs of each Python file."""

from os import listdir
import libcst as cst

def find_python_files(directory):
    """Find all of the python files in a given directory."""
    file_list = []
    # Use listdir function of os to look through all of the files in a directory
    for file in listdir(directory):
        # Parse out only Python files that have the .py extension
        if file.endswith(".py"):
            file_list.append(file)

    # Return a list of the Python files in a directory
    return file_list


def read_files(directory, file_list):
    """Read all of the python files in a directory, and turn them into Strings."""
    string_file_dict = {}
    # Iterate through all of the files in the list of Python files
    for file in file_list:
        # Create a file path with the directory and file name
        file_path = directory + "/" + file
        # Open each file
        with open(file_path, encoding="utf-8") as file:
            # Read each file
            file_string = file.read()
            # Add the name of the file and the string file to a dictionary
            string_file_dict[file] = file_string

    # Return a dictionary with the file name as the key, and file contents as value
    return string_file_dict


def generate_cast(file_strings_dict):
    """Create a CAST of each Python file using LibCST."""
    tree_dict = {}
    # Iterate through a dictionary of file names and their contents represented as a String
    for file in file_strings_dict:
        file_string = file_strings_dict[file]
        # Generate a CAST using the module parser provided by LibCST
        cast = cst.parse_module(file_string)
        # Create a dictionary with the file name and file CAST
        tree_dict[file] = cast

    # Return a dictionary with the file name as they key and its corresponding CAST as the value
    return tree_dict



"""This module counts instances of statements in Python files."""
from typing import Dict
import libcst.matchers as match

def match_imports(cast_dict):
    """A function for counting the number of if statements in a Python program."""
    imports_dictionary = {}
    # Iterate through all of the Python files in a directory
    for file in cast_dict:
        # Find CASTs for each of these files
        cast = cast_dict[file]
        # Determine number of import statements for each file
        imports_list = match.findall(cast, match.While())
        imports_dictionary[file] = len(imports_list)

    return imports_dictionary


def match_comment(cast_dict):
    """A function for counting the number of comments statements in a Python program."""
    comment_dictionary = {}
    for file in cast_dict:
        # Find CASTs for each of these files
        cast = cast_dict[file]
        # Determine number of comments for each file
        comment = match.findall(cast, match.Comment())
        comment_dictionary[file] = len(comment)

    return comment_dictionary


def count_whileloops(cast_dict):
    """A function for counting the number of while loops in a Python program."""
    while_dictionary = {}
    # Iterate through all of the Python files in a directory
    for file in cast_dict:
        # Find CASTs for each of these files
        cast = cast_dict[file]
        while_loops = match.findall(cast, match.While())
        while_dictionary[file] = len(while_loops)

    return while_dictionary


def count_forloops(cast_dict):
    """A function for counting the number of while loops in a Python program."""
    for_dictionary = {}
    # Iterate through all of the Python files in a directory
    for file in cast_dict:
        # Find CASTs for each of these files
        cast = cast_dict[file]
        # Determine number of for statements for each file
        imports = match.findall(cast, match.For())
        for_dictionary[file] = len(imports)

    return for_dictionary


def match_if_statements(cast_dict):
    """A function for counting the number of if statements in a Python program."""
    if_statements_dictionary = {}
    # Iterate through all of the Python files in a directory
    for file in cast_dict:
        # Find CASTs for each of these files
        cast = cast_dict[file]
        # Determine number of comments for each file
        ifs = match.findall(cast, match.If())
        if_statements_dictionary[file] = len(ifs)

    return if_statements_dictionary


def match_funcdefs(cast_dict):
    """A function for counting the number of function definitions in a Python program."""
    func_count = {}
    # Iterate through all of the Python files in a directory
    for file in cast_dict:
        # track the number of docstrings
        docstring_num = 0
        # Find CASTs for each of these files
        cast = cast_dict[file]
        # Determine number of import statements for each file
        funcdefs = match.findall(cast, match.FunctionDef())
        # store the number of functions
        func_count[file] = {"function" : 0, "docstring": 0}
        func_count[file]["function"] = len(funcdefs)
        # iterate and count the number of docstrings
        for node in funcdefs:
            if node.get_docstring():
                docstring_num += 1
        func_count[file]["docstring"] = docstring_num

    return func_count


def count_function_without_docstrings(func_count: Dict) -> int:
    """Find the number of functions missing a docstring.

    Args:
        func_count (Dict): function and docstring counts per file

    Returns:
        int: total number of functions - total number of docstrings
    """
    func_total = 0
    docstring_total = 0
    for file_count in func_count.values():
        func_total += file_count["function"]
        docstring_total += file_count["docstring"]
    return func_total - docstring_total


def sum_cast_dict(cast_dict):
    """A function for calculating the sums of values from dictionaries in previous functions."""
    total = 0
    # Total imports
    for file in cast_dict:
        amount = cast_dict[file]
        total += amount

    return total



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['castanet']

package_data = \
{'': ['*']}

install_requires = \
['libcst>=0.4.0,<0.5.0', 'typer[all]>=0.4.0,<0.5.0']

entry_points = \
{'console_scripts': ['castanet = main:app']}

setup_kwargs = {
    'name': 'castanet',
    'version': '1.0.0',
    'description': 'Program to identify different syntax in Python programs',
    'long_description': None,
    'author': 'Thomas Antle, Caden Hinckley, Maddy Kapfhammer, Bailey Matrascia, Nolan Thompson',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
